#************************************************************ 
#
# Component       CNF  
#
# Synopsis        Common functions
#
#
# Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
#
#************************************************************
#
# #ident "@(#)COMMON.pm 1.2 04/03/12 SMI"
#
#************************************************************

#=head1 Common functions
#
#=over 4
#
#=cut

package COMMON ;

use strict;
use VAR;

my $G_nhcmmadm = "/opt/SUNWcgha/sbin/nhcmmadm";
my $G_nhnetcfg = "/opt/SUNWcgha/sbin/nhnetcfg";

#------------------------------------------------------------------------------
#
#  convertDottedToBitmask
#
#    convert a dotted form to a bit mask
#
#------------------------------------------------------------------------------

sub convertDottedToBitmask {

  my ($dotted) = @_;

  my $mask = 0;
  foreach my $value (split('\.', $dotted)) {
    $mask = $mask * 256 + $value;
  }

  return $mask;
}

#------------------------------------------------------------------------------
#
#  convertBitmaskToDotted
#
#    convert a bit mask to a dotted form
#
#------------------------------------------------------------------------------

sub convertBitmaskToDotted {

  my ($bitmask) = @_;

  my $dotted = "";
  for (my $i=0; $i < 4; $i++) {
    if ($i == 0) {
      $dotted = sprintf("%d", $bitmask % 256);
    } else {
      $dotted = sprintf("%d.%s", $bitmask % 256, $dotted)
    }
    $bitmask = $bitmask / 256;
  }

  return $dotted;
}

#--------------------------------------------------------------
#
#  exist_physical_interface
#
#  - input interface can be an alias
#
#--------------------------------------------------------------

sub exist_physical_interface  {

  my ($interface, $ref_physical_interface) = @_;

  my @result;
  $$ref_physical_interface = (split(":",$interface))[0];
  @result = qx{/sbin/ifconfig -a};

  foreach my $line (@result) {
    if ($line =~ m/^$$ref_physical_interface:[ ]/) {
      return $TRUE;
    }
  }

  return $FALSE;
}

#--------------------------------------------------------------
#
#  get_interface_info
#
#--------------------------------------------------------------

sub get_interface_info  {

  my ($interface, $info) = @_;

  my @result;
  @result = qx{/sbin/ifconfig $interface};

  if (scalar(@result) == 0) {
    return "";
  }

  foreach my $line (@result) {
    if (($info eq "FLAGS") && ($line =~ /flags=[0-9]*<([^>]*)>/o)) {
      return $1;
    }
    elsif (($info eq "MTU") && ($line =~ /mtu ([0-9]*)/o)) {
      return $1;
    }
    elsif (($info eq "INDEX") && ($line =~ /index ([0-9]*)/o)) {
      return $1;
    }
    elsif (($info eq "INET") && ($line =~ /inet ([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*)/o)) {
      return $1;
    }
    elsif (($info eq "NETMASK") && ($line =~ /netmask ([0-9a-f]*)/o)) {
      return $1;
    }
    elsif (($info eq "BROADCAST") && ($line =~ /broadcast ([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*)/o)) {
      return $1;
    }
    elsif (($info eq "ETHER") && ($line =~ /ether ([0-9aa-f:]*)/o)) {
      return $1;
    }
  } 

  return "";
}

#------------------------------------------------------------
#
# get_node_id
#
#------------------------------------------------------------

sub get_node_id {

  my ($role) = @_;

  my $option = $role;
  if ($role eq "vicemaster") {
    $option = "vice";
  }

  my @result = qx/$G_nhcmmadm -c $option/;

  foreach my $line (@result) {
    chomp($line);
    if ($line =~ m/node_id[\s\t]*=[\s\t]([0-9]*)/) {
      return $1;
    }
  }

#=item Cannot determine node id for %s
#
#    The actual node id cannot be determined.
#    Verify that the node is booted.
#
#=cut


  LOG::printError("Cannot determine node id for %s", $role);
  exit 1;
}

#------------------------------------------------------------
#
#  set__men_addresses
#
#------------------------------------------------------------

sub set_men_addresses
  {
    my ($node_id_array, $men_ip_array) = @_;
    my $i;

    if (@$node_id_array[0] eq "")
      {
	@$node_id_array[0] = get_node_id("master") ;
      }
    if (@$node_id_array[1] eq "")
      {
	@$node_id_array[1] = get_node_id("vicemaster") ;
      }

    # method extracted from the CGTP startup script
    for ($i = 0 ; $i < 2 ; $i++) {
      my $result;
      chomp($result = qx{$G_nhnetcfg cgtp @$node_id_array[$i]}) ;
      if ($result ne "")
      {
	@$men_ip_array[$i] = (split(':', $result))[1]; ;
      } else {

#=item Cannot determine IP address for node <id>
#
#    The IP address of the node cannot be determined.
#
#=cut

	LOG::printError("Cannot determine IP address for node %d", 
			@$node_id_array[0]);
      }
    }
  }

#------------------------------------------------------------
#
#  execute a local command and display the output:
#  . if there is an error
#  . if tracing option is set
#
#  1: command
#
#------------------------------------------------------------

sub exec_cmd {
    my ($command) = @_;

    LOG::printMsg(2,"%s", $command) ;
    my @result = qx/$command 2>&1/;

    # in case of error, display output and exit
    if ($? ne 0 )
      {
      my $status = $? / 256 ;
      LOG::printMsg(0,"Error when executing: %s (status=%s)", $command, $status) ;
      foreach my $line (@result)
	{
	  LOG::printMsg(0,"  %s", $line) ;
	}
      return 0 ;
    }
    
    # trace if required
    foreach my $line (@result)
      {
	LOG::printMsg(2,"  %s", $line) ;
      }
    
    return 1 ;
}

#--------------------------------------------------------------
#
#  main
#
#--------------------------------------------------------------

#=back
#
#=cut
{
}
