#************************************************************ 
#
# Component       CNF
#
# Synopsis        Synchronization between configuration files
#
# Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
#
#************************************************************
#
# #ident "@(#)SYNC.pm 1.11 04/03/12 SMI"
#
#************************************************************

#=head1 File synchronization
#
#=over 4
#
#=cut

package SYNC ;

use strict;
use VAR;
use LOG;


#
# variables to be exported
#

my $G_pid ;

my $G_nodetag = "NODEID=" ;
my $G_filetag = "FILE=" ;
my $G_begintag = "=BEGIN" ;
my $G_endtag = "=END" ;

my @G_node_id_array = ( "", "" ) ;
my @G_men_IP = ();
my $G_node_id;

my $G_nhcmmadm = "/opt/SUNWcgha/sbin/nhcmmadm";
my $G_nhnetcfg = "/opt/SUNWcgha/sbin/nhnetcfg";



#--------------------------------------------------------------
#
#  rcp
#
#--------------------------------------------------------------

sub rcp
  {
    my ($file, $Rfile1, $Rfile2) = @_ ;
    
    $$Rfile1 = "/tmp/file1.$G_pid";
    $$Rfile2 = "/tmp/file2.$G_pid";

    if ($G_node_id_array[0] == $G_node_id) {
      COMMON::exec_cmd("/usr/bin/cp $file $$Rfile1") || return 0;
    } else {
      COMMON::exec_cmd("/usr/bin/rcp $G_men_IP[0]:$file $$Rfile1") || return 0;
      if (! -f $$Rfile1) {
	LOG::printError("Cannot retrieve %s from %s", $file, $G_men_IP[0]);
	return 0;
      }
    }
    if ($G_node_id_array[1] == $G_node_id) {
      COMMON::exec_cmd("/usr/bin/cp $file $$Rfile2") || return 0;
    } else {
      COMMON::exec_cmd("/usr/bin/rcp $G_men_IP[1]:$file $$Rfile2") || return 0;
      if (! -f $$Rfile2) {
	LOG::printError("Cannot retrieve %s from %s", $file, $G_men_IP[1]);
	return 0;
      }
    }
    return 1 ;
  }

#--------------------------------------------------------------
#
#  process_file
#
#--------------------------------------------------------------

sub process_file
  {
    my ($file, $diff) = @_ ;
    my $file1 ;
    my $file2 ;

    my $result;
    my $status;

    LOG::printMsg(0, "Processing %s ...", $file) ;

    if (!rcp($file, \$file1, \$file2)) {
      return 0;
    }

    $result = qx/diff -b $file1 $file2 2>&1/;
    $status = $? % 255;

    # in case of error, display output and exit
    if ($status == 0) {
      # no differences were found
      # check that no differences have been registered
      if ($diff ne "") {
	  LOG::printError("%s: no differences found (should be different)", $file) ;
	}
    } elsif ($status == 1 ) {
      # differences were found
      # compared with the previous differences
      if ($result ne $diff) {
	LOG::printError("%s: not synchronized", $file) ;
      }
    } else {
      LOG::printError("Diff error when comparing %s and %s", $file1, $file2);
    }
   
    unlink($file1) ;
    unlink($file2) ;
  }

#--------------------------------------------------------------
#
#  process_file_gen
#
#--------------------------------------------------------------

sub process_file_gen
  {
    my ($file) = @_ ;
    my $file1 ;
    my $file2 ;

    LOG::printMsg(0, "Processing %s ...", $file) ;

    if (! rcp($file, \$file1, \$file2)) {
      return 0;
    }

    my $result = qx/diff -b $file1 $file2 2>&1/;

    # in case of error, there is differences: copy them
    if ($? ne 0 )
      {
	print NEWSYNCFILE "$G_begintag\n" ;
	print NEWSYNCFILE $result ;
	print NEWSYNCFILE "$G_endtag\n" ;
      }
   
    unlink($file1) ;
    unlink($file2) ;
  }


#--------------------------------------------------------------
#
#  synccheck
#
#--------------------------------------------------------------

sub synccheck
  {
    my $file = "" ;
    my $diff = "" ;
    my $begin = "FALSE" ;
    my $line;

    $G_pid = $$ ;

    CONF::load_fs_conf();

    $G_node_id = CMM::get_local_node_id;
    COMMON::set_men_addresses(\@G_node_id_array, \@G_men_IP) ;

    if (!open(SYNCFILE, "<$E_syncfile")) {
      LOG::printError("error when accessing %s", $E_syncfile) ;
      exit 1;
    }
    while (<SYNCFILE>)
      {
	chomp($line = $_) ;
	LOG::printMsg(2,"%s", $line) ;

	if ($begin eq "FALSE") {
	  $line =~ s/#.*// ;
	}

	if ($line eq "") {
	  next ;
	}

	if ($line =~ m/$G_nodetag(.*)/)
	  {
	    $line =~ m/$G_nodetag[\s\t]*([0-9]*)[\s\t]*([0-9]*)/ ;
	    if (($1 eq "") or ($2 eq ""))
	    {
	      LOG::printError("invalid %s syntax", $G_nodetag) ;
	      exit 1 ;
	    }
	    LOG::printMsg(1, "read NODEID 1=%s 2=%s\n", $1, $2) ;
	    $G_node_id_array[0] = $1 ;
	    $G_node_id_array[1] = $2 ;

	    COMMON::set_men_addresses(\@G_node_id_array, \@G_men_IP) ;
	  }
	elsif ($line =~ m/$G_filetag(.*)/) 
	  {
	    if ($file ne "")
	      {
		process_file($file, $diff) ;
	      }
	    $file = $1 ;
	    $diff = "" ;
	    LOG::printMsg(2,"File to be processed: %s", $file) ;
	  }
	elsif ($line =~ m/$G_begintag/)
	  {
	    $begin = "TRUE" ;
	    $diff = "" ;
	  }
	elsif ($line =~ m/$G_endtag/)
	  {
	    $begin = "FALSE" ;
	  }
	elsif ($begin eq "TRUE")
	  {
	    $diff="$diff$_" ;
	  }
	else
	  {
	    LOG::printError("syntax error: line %s", $.) ;
	    exit 1 ;
	  }
      }
    if ($file ne "")
      {
	process_file($file, $diff) ;
      }

    close(SYNCFILE) ;
  }

#--------------------------------------------------------------
#
#  syncgen
#
#--------------------------------------------------------------

sub syncgen
  {
    my $file = "" ;
    my $begin = "FALSE" ;
    my $nodeid_created = "FALSE" ;
    my $line;

    $G_pid = $$ ;

    CONF::load_fs_conf();

    $G_node_id = CMM::get_local_node_id();
    COMMON::set_men_addresses(\@G_node_id_array, \@G_men_IP) ;

    my $newfile = "/tmp/nhadmsync.$G_pid" ;

    if (!open(SYNCFILE, "<$E_syncfile")) {
      LOG::printError("error when accessing %s", $E_syncfile) ;
      exit 1;
    }
    if (!open(NEWSYNCFILE, ">$newfile")) {
      LOG::printError("error when creating %s", $newfile) ;
      exit 1;
    }
    while (<SYNCFILE>)
      {
	chomp($line = $_) ;
	LOG::printMsg(2,"%s", $line) ;
	$line =~ s/#.*// ;

	if ($line eq "") {
	  # it's a comment, just copy it
	  print NEWSYNCFILE $_ ;
	  next ;
	}

	if ($line =~ m/$G_nodetag(.*)/)
	  {
	    $line =~ m/$G_nodetag[\s\t]*([0-9]*)[\s\t]*([0-9]*)/ ;
	    if (($1 eq "") or ($2 eq ""))
	    {
	      LOG::printError("invalid %s syntax", $G_nodetag) ;
	      exit 1 ;
	    }
	    LOG::printMsg(1, "read NODEID 1=%s 2=%s", $1, $2) ;
	    $G_node_id_array[0] = $1 ;
	    $G_node_id_array[1] = $2 ;
	  }
	elsif ($line =~ m/$G_filetag(.*)/) 
	  {
	    # when processing the first line, if NODEID line has not been created
	    # it is created
	    if ($file eq "")
	      {
		if ($nodeid_created eq "FALSE")
		  {
		    COMMON::set_men_addresses(\@G_node_id_array, \@G_men_IP) ;
		    print NEWSYNCFILE "${G_nodetag}$G_node_id_array[0] $G_node_id_array[1]\n" ;
		    $nodeid_created = "TRUE" ;
		  }
	      }
	    $file = $1 ;
	    print NEWSYNCFILE "$G_filetag$file\n" ;
	    process_file_gen($file) ;

	    LOG::printMsg(2,"File to be processed: %s", $file) ;
	  }
	elsif ($line =~ m/$G_begintag/)
	  {
	    $begin = "TRUE" ;
	  }
	elsif ($line =~ m/$G_endtag/)
	  {
	    $begin = "FALSE" ;
	  }
	elsif ($begin ne "TRUE")
	  {
	    LOG::printError("syntax error: line %s", $.) ;
	    exit 1 ;
	  }
      }

    close(SYNCFILE) ;
    close(NEWSYNCFILE) ;

    COMMON::exec_cmd("cp -p $E_syncfile $E_syncfile.bak") ;
    COMMON::exec_cmd("cp $newfile $E_syncfile") ;
    unlink($newfile) ;
  }

#=back
#
#=cut

{
}

